"""
EBAS IO File base definistions
"""

EBAS_IOFORMAT_NASA_AMES = 0
EBAS_IOFORMAT_CSV = 1
EBAS_IOFORMAT_XML = 2
EBAS_IOFORMAT_NETCDF1 = 3
EBAS_IOFORMAT_NETCDF = 4
EBAS_IOFORMAT_OPENDAP = 5

EBAS_IOSTYLE_SINGLECOLUMN = 0
EBAS_IOSTYLE_MULTICOLUMN = 1
EBAS_IOSTYLE_MULTICOLUMN_PRECIP = 2
EBAS_IOSTYLE_KEEP = 3

# bitwise options:
EBAS_IOMETADATA_OPTION_SETKEY = 1
EBAS_IOMETADATA_OPTION_ALL = \
    EBAS_IOMETADATA_OPTION_SETKEY

FLAGS_ONE_OR_ALL = 0
FLAGS_COMPRESS = 1
FLAGS_ALL = 2
FLAGS_NONE = 3
FLAGS_AS_IS = 4

SUPPRESS_SORT_VARIABLES = 1
SUPPRESS_METADATA_OCCURRENCE = 2

