"""
EBAS domain layer (entity objects and domain logic)

The ebas domain module implements the core domain logic in EBAS.

 * ::: ebas.domain.entities
       options:
           members: no
 * ::: ebas.domain.access_statistics
       options:
           members: no
 * ::: ebas.domain.masterdata
       options:
           members: no
"""

from .base import EbasDomainError, EbasDomainDbContext
